"""
Collection of one-size-fits-all wisdoms and questionable quotations
"""

fortunes = [

    # Technology #

    'The plural of Regex is "Regrets". ',

    'Why make it simple and efficient, '
    'when you can make it a huge overcomplicated mess? ',

    '''"Stable"? That's for horses. '''

    'Roses are red, violets are blue. '
    'Unterminated string literal at line 32. ',

    'A computer is much like air conditioning: '
    'useless the moment you open Windows. ',

    'We built our world without walls and fences, '
    "so naturally we don't need Windows and Gates. ",

    "object: v. to feel strong distaste; to oppose firmly.\n"
    "See also: object-oriented programming. ",

    "Just a few hours of trial and error can save "
    "countless minutes of reading the documentation. ",

    "Printing is not supported on this printer. ",

    "Required qualifications: minimal Java experience (ideally none)",

    "Configuring login daemon...\n"
    "Starting login daemon...\n"
    "Exorcising login daemon...\n"
    "System operational.",

    # Education #

    '"The doors are wide open, '
    'but we still see people crashing into the doorframe.\n" '
    '    -- NvdM on IC exams',

    'In order to save energy and help meet our goal of a carbon-neutral '
    'campus by 2025, the light at the end of the tunnel has been turned off. ',

    "Technical university: technically, it's a university. ",

    "Theoretical physicists: in theory, they're good at physics. ",

    "Don't try; just do. ",

    "If you can't, then don't. ",

    "Pay attention, I know exactly what I am talking about. ",

    "Just because [pipelining] wasn't covered in the lectures "
    "doesn't meant it won't be on the exam. ",

    "Learning by doing: I do, you learn. ",

    "There are no stupid questions, "
    "only stupid people. ",

    # Politics #

    "Abuse of authority comes as no surprise. ",

    "Money doesn't bring happiness, but neither does poverty. ",

    '"Somehow the potential to make a lot of money has become more valuable '
    'then making a lot of money lol. " -- u/h3J1e on shareholders',

    "Elect a clown, expect a circus. ",

    "Points of discussion: "
    "construction of a shed, and construction of communism. "
    "Due to lumber shortages, it was decided to skip "
    "straight to the second point. ",

    'Now is the time to save face. We hope there will be time to '
    'save other parts of the body as well. ',

    "We wanted the best, but it turned out like always. ",

    "No matter what we try to develop, we always end up with either "
    "the communist party or the Khalashnikov rifle.",

    "You can't hug a child with nuclear arms. ",

    'Experimental missile nicknamed "Civil Servant": '
    "won't work and they can't fire it",

    "We have thoroughly addressed all concerns, from A to B. ",

    "He is a man of logic and should not be taken seriously. ",

    "That is my opinion. Do not ask me why. ",

    "- A choice of what? \n"
    "- A choice of watt? \n"
    "- Yes, a choice of what? \n"
    "- No, the watts are exactly the same, "
    "the choice is who you buy them from. ",

    "- How long are you in for?\n"
    "- Three years\n"
    "- What for?\n"
    "- For nothing!\n"
    "- Do not try to play us for fools, "
    'everyone knows that "for nothing" is 5 years. ',

    # Malapropisms #

    "Whatever bloats your goat! ",

    # Engineering #

    "Instructions for use: do not. ",

    "Square peg. Round hole. Get it done. ",

    "Emergency phone not installed. "
    "Please do not have an emergency at this location. ",

    'The difference between practice and theory '
    'is often bigger in practice than in theory. ',

    'No keyboard detected. Press F12 to continue.',

    "Forbidden to remove boulder. "
    "Without it, lift does not carry children. ",

    'Anything labeled "this end towards enemy" is dangerous at both ends. ',

    "It works all the time 60% of the time. ",

    # Resilience #

    "Even with everyone's help, you still have to do everything on your own. ",

    "YOU GOTTA BLAME SOMEONE, RIGHT? BLAME YOURSELF",

    "This time it's the same, but also a little bit different. ",

    "It's not a superiority complex if you're actually superior. ",

    "It's not paranoia if someone's actually out to get you. ",

    "The horrors persist, but so do I",

    "New liver, same eagles. ",

    "It is what it is. ",

    "Life is a joke; don't be the punchline. ",

    "Just because you can survive in harsh conditions "
    "doesn't mean you should have to. ",

    "Tip: spice up your panic attack with a harmonica. ",

    # Misc "

    "The best storytellers are those who have a story to tell. ",

    "CAUTION: Gate swings both ways HARD",

    "On one hand, you've got five oranges. On the other hand, you've got "
    "three apples. Conclusion: you have big hands. ",

    "When you are dead, you do not suffer. "
    "The pain is only felt by those around you. "
    "The same happens when you are stupid. ",

    "He's suffered for his art and now it's our turn. ",

    "i may be cringe but youre mean and thats worse",

    "Qaulifications: former child",

    '"Whenever a door closes, another one surely opens"\n'
    '    -- My uncle (wise man, but a terrible cabinet maker)',

    "Present day, present time. ",

    "As above, so below. ",

    "I am not insulting you, I am *describing* you. ",

    "I hope this email finds you before I do. ",

    "To whom it WILL concern: ",

    ]

